use super::models;
use crate::api::core::models::Error;
use crate::core;
use std::collections::HashMap;

///
pub struct Api {
  client: reqwest::Client,
  api_config: core::config::ApiConfig,
}

impl std::fmt::Debug for Api {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Api of api.kgon.identity").finish()
  }
}

impl Api {
  ///
  pub fn new(api_config: core::config::ApiConfig) -> Api {
    Api {
      client: reqwest::Client::new(),
      api_config,
    }
  }

  ///
  pub async fn list_bettings(
    &self,
    data: models::ListBettingsRequest,
  ) -> Result<models::ListBettingsResponse, Error> {
    let mut params = HashMap::new();

    if let Some(vendor_key) = data.vendor_key {
      params.insert("vendorKey", vendor_key);
    }
    if let Some(sdate) = data.sdate {
      params.insert("sdate", sdate);
    }
    if let Some(edate) = data.edate {
      params.insert("edate", edate);
    }
    if let Some(group_key) = data.group_key {
      params.insert("groupKey", group_key);
    }
    if let Some(username) = data.username {
      params.insert("username", username);
    }

    params.insert("limit", data.limit.to_string());

    let res = match self
      .client
      .post(format!("{}/transaction", self.api_config.k_url))
      .header(reqwest::header::ACCEPT, "application/json")
      .header(
        reqwest::header::CONTENT_TYPE,
        "application/x-www-form-urlencoded",
      )
      .header("k-secret", self.api_config.k_secret.as_str())
      .header("k-username", self.api_config.k_username.as_str())
      .send()
      .await
    {
      Ok(res) => res,
      Err(e) => {
        return Err(Error {
          code: -1,
          msg: Some(e.to_string()),
        });
      }
    };

    match res.status() {
      reqwest::StatusCode::OK => match res.json::<models::_ListBettingsResponse>().await {
        Ok(r) => {
          if r.code != 0 {
            return Err(Error {
              code: r.code,
              msg: r.msg,
            });
          }

          let transactions = match r.transactions {
            Some(v) => v,
            None => vec![],
          };

          Ok(models::ListBettingsResponse { transactions })
        }
        Err(e) => Err(Error {
          code: -1,
          msg: Some(e.to_string()),
        }),
      },
      _ => Err(Error {
        code: -1,
        msg: None,
      }),
    }
  }

  ///
  pub async fn get_gragmatic_betting(
    &self,
    data: models::GetPragmaticBettingRequest,
  ) -> Result<models::GetPragmaticBettingResponse, Error> {
    let mut params = HashMap::new();

    params.insert("id", data.id);

    let res = match self
      .client
      .post(format!("{}/pragmatic/history", self.api_config.k_url))
      .header(reqwest::header::ACCEPT, "application/json")
      .header(
        reqwest::header::CONTENT_TYPE,
        "application/x-www-form-urlencoded",
      )
      .header("k-secret", self.api_config.k_secret.as_str())
      .header("k-username", self.api_config.k_username.as_str())
      .form(&params)
      .send()
      .await
    {
      Ok(res) => res,
      Err(e) => {
        return Err(Error {
          code: -1,
          msg: Some(e.to_string()),
        });
      }
    };

    match res.status() {
      reqwest::StatusCode::OK => match res.json::<models::_GetPragmaticBettingResponse>().await {
        Ok(r) => {
          if r.code != 0 {
            return Err(Error {
              code: r.code,
              msg: r.msg,
            });
          }

          let url = match r.url {
            Some(v) => v,
            None => "".to_string(),
          };

          Ok(models::GetPragmaticBettingResponse { url })
        }
        Err(e) => Err(Error {
          code: -1,
          msg: Some(e.to_string()),
        }),
      },
      _ => Err(Error {
        code: -1,
        msg: None,
      }),
    }
  }

  ///
  pub async fn get_evolution_betting_detail(
    &self,
    data: models::GetEvolutionBettingDetailRequest,
  ) -> Result<models::GetEvolutionBettingDetailResponse, Error> {
    let mut params = HashMap::new();

    params.insert("ids", data.ids);

    let res = match self
      .client
      .post(format!(
        "{}/evolution/transaction/detail",
        self.api_config.k_url
      ))
      .header(reqwest::header::ACCEPT, "application/json")
      .header(
        reqwest::header::CONTENT_TYPE,
        "application/x-www-form-urlencoded",
      )
      .header("k-secret", self.api_config.k_secret.as_str())
      .header("k-username", self.api_config.k_username.as_str())
      .form(&params)
      .send()
      .await
    {
      Ok(res) => res,
      Err(e) => {
        return Err(Error {
          code: -1,
          msg: Some(e.to_string()),
        });
      }
    };

    match res.status() {
      reqwest::StatusCode::OK => match res
        .json::<models::_GetEvolutionBettingDetailResponse>()
        .await
      {
        Ok(r) => {
          if r.code != 0 {
            return Err(Error {
              code: r.code,
              msg: r.msg,
            });
          }

          let transactions = match r.transactions {
            Some(v) => v,
            None => vec![],
          };

          Ok(models::GetEvolutionBettingDetailResponse { transactions })
        }
        Err(e) => Err(Error {
          code: -1,
          msg: Some(e.to_string()),
        }),
      },
      _ => Err(Error {
        code: -1,
        msg: None,
      }),
    }
  }

  ///
  pub async fn get_statistic(
    &self,
    data: models::GetStatisticRequest,
  ) -> Result<models::GetStatisticResponse, Error> {
    let mut params = HashMap::new();

    if let Some(vendor_key) = data.vendor_key {
      params.insert("vendorKey", vendor_key);
    }
    if let Some(group_key) = data.group_key {
      params.insert("groupKey", group_key);
    }
    if let Some(date) = data.date {
      params.insert("date", date);
    }

    let res = match self
      .client
      .post(format!("{}/statistic", self.api_config.k_url))
      .header(reqwest::header::ACCEPT, "application/json")
      .header(
        reqwest::header::CONTENT_TYPE,
        "application/x-www-form-urlencoded",
      )
      .header("k-secret", self.api_config.k_secret.as_str())
      .header("k-username", self.api_config.k_username.as_str())
      .form(&params)
      .send()
      .await
    {
      Ok(res) => res,
      Err(e) => {
        return Err(Error {
          code: -1,
          msg: Some(e.to_string()),
        });
      }
    };

    match res.status() {
      reqwest::StatusCode::OK => match res.json::<models::_GetStatisticResponse>().await {
        Ok(r) => {
          if r.code != 0 {
            return Err(Error {
              code: r.code,
              msg: r.msg,
            });
          }

          let data = match r.data {
            Some(v) => v,
            None => "".to_string(),
          };

          Ok(models::GetStatisticResponse { data })
        }
        Err(e) => Err(Error {
          code: -1,
          msg: Some(e.to_string()),
        }),
      },
      _ => Err(Error {
        code: -1,
        msg: None,
      }),
    }
  }
}
